from flask import Flask
from flask_sqlalchemy import SQLAlchemy

from management.data import shared
from management.data.data_test import init_test_data
from management.data.data_dev import init_dev_data
from management.data.data_prod import init_prod_data
from portal.database.models import Course, Role, User
from portal.service import general


class DataManagement(object):
    def __init__(self, app: Flask, db: SQLAlchemy):
        """Creates instance of th data manager
        Args:
            app(Flask): Flask application
            db(SQLAlchemy): SqlAlchemy Db instance
        """
        self.app = app
        self.db = db
        self.creator = shared.DataFactory(self.db)

    def reset_db(self):
        """Resets the database
        """
        self.db.drop_all(app=self.app)
        self.db.create_all(app=self.app)

    def init_data(self, env=None):
        """Init sample data
        Args:
            env(str): Name of the environment
        """
        env = env
        if env == 'prod':
            init_prod_data(db=self.db, app=self.app)
        if env == 'test':
            init_test_data(db=self.db, app=self.app)
        else:
            init_dev_data(db=self.db, app=self.app)

    def delete_user(self, name):
        """Deletes user
        Args:
            name(str): name of the user
        """
        with self.app.app_context():
            user = general.find_user(name)
            general.delete_entity(user)

    def update_users_password(self, name: str, password: str) -> User:
        """Updates user's password
        Args:
            name(str): Name of the user
            password(str): User's password
        Returns(User): Updated user
        """
        with self.app.app_context():
            user = general.find_user(name)
            user.set_password(password=password)
            self.db.session.add(user)
            self.db.session.commit()
        return user

    def create_admin_user(self, name: str, password: str) -> User:
        """Creates new admin user with username
        Args:
            name(str): Username of the user
            password(str): User's password
        Returns(User): Created user
        """
        with self.app.app_context():
            user = self.creator.create_user(
                username=name,
                name=name,
                admin=True,
                password=password
                )
            self.db.session.commit()
        return user

    def create_course(self, name: str) -> Course:
        """Creates course with associated roles
        Args:
            name(str): Codename of the course
        Returns(Course): Course instance
        """
        with self.app.app_context():
            course = self.creator.scaffold_course(codename=name)
            self.db.session.commit()
        return course

    def create_role(self, course_name: str, role_type: str, name: str) -> Role:
        """Creates role in the course based on type
        Args:
            name(str): Name of the role
            course_name(str): Name of the course
            role_type(str): Type of the role
        Returns(Role): Created role
        """
        with self.app.app_context():
            course = general.find_course(course_name)
            role = self.creator.scaffold_role(course, role_type=role_type, name=name)
            self.db.session.commit()
        return role

    def list_users(self):
        with self.app.app_context():
            for user in User.query.all():
                print(f"{user.username} ({user.id}) - {user.created_at}")

