import logging
from flask import request
from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal.rest import rest_helpers
from portal.rest.schemas import course_import_schema, course_schema, courses_schema, users_schema
from portal.service import permissions
from portal.service.auth import find_client
from portal.service.courses import copy_course, create_course, delete_course, find_all_courses, \
    get_users_filtered, update_course, update_notes_token
from portal.service.errors import ForbiddenError, PortalAPIError
from portal.service.filters import filter_course_dump
from portal.service.general import find_course
from portal.service.permissions import check_client, require_client

courses_namespace = Namespace('courses')
log = logging.getLogger(__name__)


@courses_namespace.route('')
class CourseList(Resource):
    @jwt_required
    #@courses_namespace.response(200, 'Courses list', model=courses_schema)
    @courses_namespace.response(403, 'Not allowed to see courses')
    def get(self):
        client = find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        courses_list = find_all_courses()
        return courses_schema.dump(courses_list)

    @jwt_required
    #@courses_namespace.response(200, 'Created course', model=course_schema)
    #@courses_namespace.response(403, 'Not allowed to create course', model=course_schema)
    def post(self):
        client = find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(course_schema, resource='course', action='create')
        new_course = create_course(**data)
        return course_schema.dump(new_course)[0], 201


@courses_namespace.route('/<string:cid>')
@courses_namespace.param('cid', 'Course id')
@courses_namespace.response(404, 'Course not found')
class CourseResource(Resource):

    @jwt_required
    #@courses_namespace.response(200, 'Course found', model=course_schema)
    @courses_namespace.response(403, 'Not allowed to see course')
    def get(self, cid: str):
        client = find_client()
        course = find_course(cid)
        # authorization
        if (check_client(client=client,
                         course=course,
                         permissions=['view_course_full'])):
            return course_schema.dump(course)

        elif (check_client(client=client,
                           course=course,
                           permissions=['view_course_limited'])):
            dump = course_schema.dump(course)
            filtered_course = filter_course_dump(course, dump.data, client)
            return filtered_course

        raise ForbiddenError(uid=client.id)

    @jwt_required
    @courses_namespace.response(204, 'Course deleted')
    @courses_namespace.response(403, 'Not allowed to delete course')
    def delete(self, cid: str):
        client = find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        course = find_course(cid)
        delete_course(course)
        return '', 204

    @jwt_required
    @courses_namespace.response(204, 'Course updated')
    @courses_namespace.response(403, 'Not allowed to update course')
    def put(self, cid: str):
        client = find_client()
        course = find_course(cid)
        # authorization
        if not (check_client(client=client, course=course, permissions=['update_course'])):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            schema=course_schema, action='update', resource='course'
            )
        update_course(course, data)
        return '', 204


@courses_namespace.route('/<string:cid>/notes_access_token')
@courses_namespace.param('cid', 'Course id')
@courses_namespace.response(404, 'Course not found')
class CourseNotesToken(Resource):
    @jwt_required
    @courses_namespace.response(403, 'Not allowed to manipulate with course\'s notes access token')
    @courses_namespace.response(200, 'Course notes token')
    def get(self, cid):
        client = find_client()
        course = find_course(cid)
        # authorization
        perm = ['handle_notes_access_token']
        require_client(client=client, course=course, permissions=perm)

        return course.notes_access_token

    @jwt_required
    @courses_namespace.response(204, 'Course\'s notes access token updated')
    @courses_namespace.response(403, 'Not allowed to update course\'s notes access token')
    def put(self, cid):
        client = find_client()
        course = find_course(cid)
        # authorization
        perm = ['handle_notes_access_token']
        require_client(client=client, course=course, permissions=perm)

        json_data = rest_helpers.require_data(action='update_notes_token', resource='course')

        update_notes_token(course, json_data)
        return '', 204


@courses_namespace.route('/<string:cid>/import')
@courses_namespace.param('cid', 'Course id')
@courses_namespace.response(404, 'Course not found')
class CourseImport(Resource):
    @jwt_required
    #@courses_namespace.response(200, 'Course has been imported', model=course_schema)
    @courses_namespace.response(403, 'Not allowed to import course')
    @courses_namespace.response(400, 'Cannot import course to itself.')
    def put(self, cid: str):
        client = find_client()
        course = find_course(cid)
        # authorization
        require_client(client=client, course=course, permissions=['update_course'])

        data = rest_helpers.parse_request_data(
            course_import_schema, action='import', resource='course'
            )
        source_course = find_course(data['source_course'])
        if source_course == course:
            raise PortalAPIError(400, f'[IMPORT] Cannot import course to itself. (id: {cid})')

        config = data['config']

        copied_course = copy_course(source_course, course, config)
        return course_schema.dump(copied_course)[0]


@courses_namespace.route('/<string:cid>/users')
@courses_namespace.param('cid', 'Course id')
@courses_namespace.response(404, 'Course not found')
class CourseUsers(Resource):

    @jwt_required
    #@courses_namespace.response(200, 'Users in the course', model=users_schema)
    @courses_namespace.response(403, 'Not allowed to see users in the course')
    def get(self, cid):
        client = find_client()
        course = find_course(cid)
        group_ids = request.args.getlist('group')
        role_ids = request.args.getlist('role')
        # authorization
        perm = ['view_course_full']
        require_client(client=client, course=course, permissions=perm)
        users = get_users_filtered(course, group_ids, role_ids)
        return users_schema.dump(users)[0]
