"""
Permissions service
"""

import logging
from typing import List, Union

from portal.database.models import Component, Course, User
from portal.service import general
from portal.service.errors import ForbiddenError

FILTER_PERMISSION_ATTRS = ['id', 'role_id', 'role', 'description', 'created_at', 'updated_at']

log = logging.getLogger(__name__)


def require_client(client: Union[Component, User], course: Course, permissions=None, check=None):
    """Require clients permissions for a course

   Args:
       client(Component, User): Client instance
       course(Course): Course instance
       permissions: List of permissions required
       check: Check that will be called

   Returns(bool): True if checks has passed
   """
    check = check or check_client
    if not check(client=client, course=course, permissions=permissions):
        raise ForbiddenError(client.id)


def check_client(client: Union[Component, User], course: Course, permissions=None):
    """Checks clients permissions for a course

   Args:
       client(Component or User): Client instance
       course(Course): Course instance
       *permissions: List of permissions required

   Returns(bool): True if checks has passed
   """
    check = (check_component(component=client) or
             check_user(user=client, course=course, permissions=permissions))

    return check


def check_component(component: Component) -> bool:
    """Checks component permissions for a course

    Args:
        component(Component): Component instance
    Returns(bool): True if checks has passed
    """
    if not isinstance(component, Component):
        return False
    return True


def check_user(user: User, course: Course, permissions: List) -> bool:
    """Checks a user's permissions based on his roles in the course

    Args:
        user(User): User instance
        course(Course): Course instance
        permissions(List): Permissions list of lists

    Returns(bool): True if checks pass
    """
    log.debug(f"[PERM] User {user.username} in course {course.codename}: {permissions}")
    if not isinstance(user, User):
        return False
    if check_sysadmin(user):
        return True

    effective_permissions = effective_permissions_for_course(user, course=course)
    if not effective_permissions:
        return False

    def __resolve_permissions(conjunction):
        if isinstance(conjunction, str):
            conjunction = [conjunction]
        return all(effective_permissions.get(p, False) for p in conjunction)

    res = any(__resolve_permissions(conjunction) for conjunction in permissions)

    log.debug(f"[PERM] {permissions} for {user.username}: {res}")
    return res


def check_sysadmin(user: User) -> bool:
    """Checks whether the user is sysadmin

    Args:
        user(User): User instance

    Returns(bool): True if checks pass
    """
    if not isinstance(user, User):
        return False
    return user.is_admin


def require_sysadmin(user: User):
    """Requires user to be sysadmin

    Args:
        user(User): User instance
    """
    if not check_sysadmin(user):
        raise ForbiddenError(user.id)


def require_any_check(client, checks: list):
    """Requires any check in the list of checks to pass
    Args:
        client:
        checks(list): List of checks
    Returns:
    """
    if not any(checks):
        raise ForbiddenError(client.id)


def get_effective_permissions(user: User, course_id: str = None) -> dict:
    """Gets effective permissions for a user in course. If no course is specified, returns the user's
    effective permissions in all courses he is a part of.

    Args:
        user(User): User instance
        course_id(str): Course ID

    Returns(dict): Effective permissions for the user. Keys are course IDs, values
    dictionaries of permissions with their values.
    """
    course = general.find_course(course_id, throws=False)
    courses = [course] if course else user.courses
    return build_effective_permissions(user, *courses)


def build_effective_permissions(user: User, *courses) -> dict:
    """Builds effective permissions for the User in a list of courses

    Args:
        user(User): User instance
        *courses: List of courses

    Returns(dict): Effective permissions
    """
    result = {}
    for course in courses:
        result[course.id] = effective_permissions_for_course(user, course=course)
    return result


def effective_permissions_for_course(user: User, course: Course) -> dict:
    """Extracts effective permissions for user in one course

    Args:
        user(User): User instance
        course(Course): Course instance

    Returns(dict): Effective permissions dictionary
    """
    permissions = user.get_permissions_for_course(course)
    result = {}
    for permission in permissions:
        for key, value in vars(permission).items():
            if not key.startswith("_") and key not in FILTER_PERMISSION_ATTRS:
                result[key] = result.get(key) or value
    log.debug(
        f"[PERM] Effective permissions: {user.username} in course {course.codename}: {result}"
        )
    return result
