import json

from portal.database.models import Course, Review, ReviewItem, Submission, SubmissionState
from portal.service import general
from . import utils


def create_submission(client) -> Submission:
    cpp = Course.query.filter_by(codename="TestCourse1").first()
    p = cpp.projects[0]

    request_dict = {
        "project_params": "data for Kontr",
        "file_params": {
            "source": {
                "type": "git",
                "url": "https://gitlab.fi.muni.cz/xkompis/test-hello-world.git",
                "branch": "master",
                "checkout": "master"
                }
            }
        }
    request_json = json.dumps(request_dict, cls=utils.DateTimeEncoder)

    response = utils.make_request(client, f"/courses/{cpp.codename}/projects/{p.name}/submissions",
                                  data=request_json,
                                  headers={"content-type": "application/json"}, method='post')
    new_submission = utils.extract_data(response)
    return general.find_submission(new_submission['id'])


# missing tests for working with zip
def test_read(client):
    submissions = Submission.query.all()
    s = submissions[0]
    response = utils.make_request(client, f'/submissions/{s.id}', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    submission = utils.extract_data(response)
    utils.assert_submission_in(submissions, submission)


def test_delete(client):
    submissions = Submission.query.all()
    submissions_len = len(submissions)
    s = submissions[0]
    assert s.review
    reviews_len = len(Review.query.all())
    response = utils.make_request(client, f'/submissions/{s.id}', method='delete')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    submissions_updated = Submission.query.all()
    assert len(submissions_updated) == submissions_len - 1
    assert len(Review.query.all()) == reviews_len - 1


def test_read_state(client):
    submissions = Submission.query.all()
    s = submissions[0]
    response = utils.make_request(client, f'/submissions/{s.id}/state', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    state = json.loads(str(response.get_data().decode("utf-8")))['state']

    assert state == s.state.name


# needs more tests for state transitions
def test_update_state(client):
    submissions = Submission.query.all()
    s = submissions[0]
    request_dict = {
        "state": "CANCELLED"
        }
    request_json = json.dumps(request_dict)

    response = utils.make_request(client, f'/submissions/{s.id}/state', data=request_json,
                                  headers={"content-type": "application/json"}, method='put')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    s_updated = Submission.query.all()[0]

    assert s_updated.state == SubmissionState.CANCELLED


# does not pass - needs a Storage mock or config
def test_resubmit(client):
    submission = create_submission(client=client)

    request_dict = {
        "note": "resubmit reason"
        }
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f'/submissions/{submission.id}/resubmit',
                                  data=request_json,
                                  headers={"content-type": "application/json"}, method='post')

    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    submission = utils.extract_data(response)

    submissions_updated = Submission.query.all()
    utils.assert_submission_in(submissions_updated, submission[0])


def test_read_review(client):
    submissions = Submission.query.all()
    s = submissions[0]
    assert s.review

    response = utils.make_request(client, f'/submissions/{s.id}/review', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    review = utils.extract_data(response)
    utils.assert_review(expected=s.review, actual=review)


def test_create_review(client):
    s = Submission.query.filter_by(state=SubmissionState.IN_PROGRESS).first()
    assert s
    len_reviews = len(Review.query.all())
    len_review_items = len(ReviewItem.query.all())

    request_dict = {
        "review_items": [
            {
                "file": "src/main.c",
                "line": 10,
                "content": "This is wrong!"
                },
            {
                "file": "src.main.c",
                "line": 15,
                "content": "Nice job!"
                }
            ]
        }

    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f'/submissions/{s.id}/review', data=request_json,
                                  headers={"content-type": "application/json"}, method='post')

    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    review = utils.extract_data(response)
    s_updated = Submission.query.filter_by(project_id=s.project_id,
                                           state=SubmissionState.IN_PROGRESS).first()
    assert len(Review.query.all()) == len_reviews + 1
    assert len(ReviewItem.query.all()) == len_review_items + 2
    utils.assert_review(s_updated.review, review)
